@@ -10,11 +10,12 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printlinear.mli,v 1.4 1999/11/17 18:56:35 xleroy Exp $ *)
+(* $Id: printlinear.mli,v 1.5 2000/04/21 08:10:35 weis Exp $ *)
 
 (* Pretty-printing of linearized machine code *)
 
+open Format
 open Linearize
 
-val instr: instruction -> unit
-val fundecl: fundecl -> unit
+val instr: formatter -> instruction -> unit
+val fundecl: formatter -> fundecl -> unit
