@@ -10,14 +10,14 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: translclass.mli,v 1.10 2004/05/26 11:10:50 garrigue Exp $ *)
+(* $Id: translclass.mli,v 1.12 2006/04/05 02:28:12 garrigue Exp $ *)
 
 open Typedtree
 open Lambda
 
-val dummy_class : lambda -> lambda
 val transl_class :
-  Ident.t list -> Ident.t -> int -> string list -> class_expr -> lambda;;
+  Ident.t list -> Ident.t ->
+  int -> string list -> class_expr -> Asttypes.virtual_flag -> lambda;;
 
 type error = Illegal_class_expr | Tags of string * string
 
