@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: major_gc.c,v 1.54.2.1 2004/07/03 10:00:59 doligez Exp $ */
+/* $Id: major_gc.c,v 1.58.10.2 2007/11/26 16:11:49 doligez Exp $ */
 
 #include <limits.h>
 
@@ -29,8 +29,8 @@
 #include "roots.h"
 #include "weak.h"
 
-unsigned long caml_percent_free;
-long caml_major_heap_increment;
+uintnat caml_percent_free;
+intnat caml_major_heap_increment;
 CAMLexport char *caml_heap_start, *caml_heap_end;
 CAMLexport page_table_entry *caml_page_table;
 asize_t caml_page_low, caml_page_high;
@@ -41,29 +41,35 @@ static value *gray_vals_cur, *gray_vals_end;
 static asize_t gray_vals_size;
 static int heap_is_pure;   /* The heap is pure if the only gray objects
                               below [markhp] are also in [gray_vals]. */
-unsigned long caml_allocated_words;
-unsigned long caml_dependent_size, caml_dependent_allocated;
+uintnat caml_allocated_words;
+uintnat caml_dependent_size, caml_dependent_allocated;
 double caml_extra_heap_resources;
-unsigned long caml_fl_size_at_phase_change = 0;
+uintnat caml_fl_size_at_phase_change = 0;
 
 extern char *caml_fl_merge;  /* Defined in freelist.c. */
 
 static char *markhp, *chunk, *limit;
 
-static int gc_subphase;     /* Subphase_main, Subphase_weak, Subphase_final */
+static int gc_subphase;     /* Subphase_main Subphase_weak[12] Subphase_final */
 #define Subphase_main 10
-#define Subphase_weak 11
-#define Subphase_final 12
+#define Subphase_weak1 11
+#define Subphase_weak2 12
+#define Subphase_final 13
 static value *weak_prev;
 
+#ifdef DEBUG
+static unsigned long major_gc_counter = 0;
+#endif
+
 static void realloc_gray_vals (void)
 {
   value *new;
 
   Assert (gray_vals_cur == gray_vals_end);
   if (gray_vals_size < caml_stat_heap_size / 128){
-    caml_gc_message (0x08, "Growing gray_vals to %luk bytes\n",
-                     (long) gray_vals_size * sizeof (value) / 512);
+    caml_gc_message (0x08, "Growing gray_vals to %"
+                           ARCH_INTNAT_PRINTF_FORMAT "uk bytes\n",
+                     (intnat) gray_vals_size * sizeof (value) / 512);
     new = (value *) realloc ((char *) gray_vals,
                              2 * gray_vals_size * sizeof (value));
     if (new == NULL){
@@ -85,12 +91,22 @@ static void realloc_gray_vals (void)
 void caml_darken (value v, value *p /* not used */)
 {
   if (Is_block (v) && Is_in_heap (v)) {
-    if (Tag_val(v) == Infix_tag) v -= Infix_offset_val(v);
-    CAMLassert (!Is_blue_val (v));
-    if (Is_white_val (v)){
-      Hd_val (v) = Grayhd_hd (Hd_val (v));
-      *gray_vals_cur++ = v;
-      if (gray_vals_cur >= gray_vals_end) realloc_gray_vals ();
+    header_t h = Hd_val (v);
+    tag_t t = Tag_hd (h);
+    if (t == Infix_tag){
+      v -= Infix_offset_val(v);
+      h = Hd_val (v);
+      t = Tag_hd (h);
+    }
+    CAMLassert (!Is_blue_hd (h));
+    if (Is_white_hd (h)){
+      if (t < No_scan_tag){
+        Hd_val (v) = Grayhd_hd (h);
+        *gray_vals_cur++ = v;
+        if (gray_vals_cur >= gray_vals_end) realloc_gray_vals ();
+      }else{
+        Hd_val (v) = Blackhd_hd (h);
+      }
     }
   }
 }
@@ -105,11 +121,12 @@ static void start_cycle (void)
   gc_subphase = Subphase_main;
   markhp = NULL;
 #ifdef DEBUG
+  ++ major_gc_counter;
   caml_heap_check ();
 #endif
 }
 
-static void mark_slice (long work)
+static void mark_slice (intnat work)
 {
   value *gray_vals_ptr;  /* Local copy of gray_vals_cur */
   value v, child;
@@ -117,6 +134,7 @@ static void mark_slice (long work)
   mlsize_t size, i;
 
   caml_gc_message (0x40, "Marking %ld words\n", work);
+  caml_gc_message (0x40, "Subphase = %ld\n", gc_subphase);
   gray_vals_ptr = gray_vals_cur;
   while (work > 0){
     if (gray_vals_ptr > gray_vals){
@@ -178,27 +196,27 @@ static void mark_slice (long work)
       chunk = caml_heap_start;
       markhp = chunk;
       limit = chunk + Chunk_size (chunk);
-    }else if (gc_subphase == Subphase_main){
-      /* The main marking phase is over.  Start removing weak pointers to
-         dead values. */
-      gc_subphase = Subphase_weak;
-      weak_prev = &caml_weak_list_head;
-    }else if (gc_subphase == Subphase_weak){
-      value cur, curfield;
-      mlsize_t sz, i;
-      header_t hd;
+    }else{
+      switch (gc_subphase){
+      case Subphase_main: {
+        /* The main marking phase is over.  Start removing weak pointers to
+           dead values. */
+        gc_subphase = Subphase_weak1;
+        weak_prev = &caml_weak_list_head;
+      }
+        break;
+      case Subphase_weak1: {
+        value cur, curfield;
+        mlsize_t sz, i;
+        header_t hd;
 
-      cur = *weak_prev;
-      if (cur != (value) NULL){
-        hd = Hd_val (cur);
-        if (Color_hd (hd) == Caml_white){
-          /* The whole array is dead, remove it from the list. */
-          *weak_prev = Field (cur, 0);
-        }else{
+        cur = *weak_prev;
+        if (cur != (value) NULL){
+          hd = Hd_val (cur);
           sz = Wosize_hd (hd);
           for (i = 1; i < sz; i++){
             curfield = Field (cur, i);
-           weak_again:
+          weak_again:
             if (curfield != caml_weak_none
                 && Is_block (curfield) && Is_in_heap (curfield)){
               if (Tag_val (curfield) == Forward_tag){
@@ -219,33 +237,58 @@ static void mark_slice (long work)
             }
           }
           weak_prev = &Field (cur, 0);
+          work -= Whsize_hd (hd);
+        }else{
+          /* Subphase_weak1 is done.  Start removing dead weak arrays. */
+          gc_subphase = Subphase_weak2;
+          weak_prev = &caml_weak_list_head;
         }
-        work -= Whsize_hd (hd);
-      }else{
-        /* Subphase_weak is done.  Handle finalised values. */
-        gray_vals_cur = gray_vals_ptr;
-        caml_final_update ();
-        gray_vals_ptr = gray_vals_cur;
-        gc_subphase = Subphase_final;
       }
-    }else{
-      Assert (gc_subphase == Subphase_final);
-      /* Initialise the sweep phase. */
-      gray_vals_cur = gray_vals_ptr;
-      caml_gc_sweep_hp = caml_heap_start;
-      caml_fl_init_merge ();
-      caml_gc_phase = Phase_sweep;
-      chunk = caml_heap_start;
-      caml_gc_sweep_hp = chunk;
-      limit = chunk + Chunk_size (chunk);
-      work = 0;
-      caml_fl_size_at_phase_change = caml_fl_cur_size;
+        break;
+      case Subphase_weak2: {
+        value cur;
+        header_t hd;
+
+        cur = *weak_prev;
+        if (cur != (value) NULL){
+          hd = Hd_val (cur);
+          if (Color_hd (hd) == Caml_white){
+            /* The whole array is dead, remove it from the list. */
+            *weak_prev = Field (cur, 0);
+          }else{
+            weak_prev = &Field (cur, 0);
+          }
+          work -= 1;
+        }else{
+          /* Subphase_weak2 is done.  Handle finalised values. */
+          gray_vals_cur = gray_vals_ptr;
+          caml_final_update ();
+          gray_vals_ptr = gray_vals_cur;
+          gc_subphase = Subphase_final;
+        }
+      }
+        break;
+      case Subphase_final: {
+        /* Initialise the sweep phase. */
+        gray_vals_cur = gray_vals_ptr;
+        caml_gc_sweep_hp = caml_heap_start;
+        caml_fl_init_merge ();
+        caml_gc_phase = Phase_sweep;
+        chunk = caml_heap_start;
+        caml_gc_sweep_hp = chunk;
+        limit = chunk + Chunk_size (chunk);
+        work = 0;
+        caml_fl_size_at_phase_change = caml_fl_cur_size;
+      }
+        break;
+      default: Assert (0);
+      }
     }
   }
   gray_vals_cur = gray_vals_ptr;
 }
 
-static void sweep_slice (long work)
+static void sweep_slice (intnat work)
 {
   char *hp;
   header_t hd;
@@ -294,10 +337,10 @@ static void sweep_slice (long work)
    [howmuch] is the amount of work to do, 0 to let the GC compute it.
    Return the computed amount of work to do.
  */
-long caml_major_collection_slice (long howmuch)
+intnat caml_major_collection_slice (intnat howmuch)
 {
   double p, dp;
-  long computed_work;
+  intnat computed_work;
   /*
      Free memory at the start of the GC cycle (garbage + free list) (assumed):
                  FM = caml_stat_heap_size * caml_percent_free
@@ -343,17 +386,21 @@ long caml_major_collection_slice (long howmuch)
   if (p < dp) p = dp;
   if (p < caml_extra_heap_resources) p = caml_extra_heap_resources;
 
-  caml_gc_message (0x40, "allocated_words = %lu\n", caml_allocated_words);
-  caml_gc_message (0x40, "extra_heap_resources = %luu\n",
-                   (unsigned long) (caml_extra_heap_resources * 1000000));
-  caml_gc_message (0x40, "amount of work to do = %luu\n",
-                   (unsigned long) (p * 1000000));
+  caml_gc_message (0x40, "allocated_words = %"
+                         ARCH_INTNAT_PRINTF_FORMAT "u\n",
+                   caml_allocated_words);
+  caml_gc_message (0x40, "extra_heap_resources = %"
+                         ARCH_INTNAT_PRINTF_FORMAT "uu\n",
+                   (uintnat) (caml_extra_heap_resources * 1000000));
+  caml_gc_message (0x40, "amount of work to do = %"
+                         ARCH_INTNAT_PRINTF_FORMAT "uu\n",
+                   (uintnat) (p * 1000000));
 
   if (caml_gc_phase == Phase_mark){
-    computed_work = 2 * (long) (p * Wsize_bsize (caml_stat_heap_size) * 100
+    computed_work = 2 * (intnat) (p * Wsize_bsize (caml_stat_heap_size) * 100
                                 / (100 + caml_percent_free));
   }else{
-    computed_work = 2 * (long) (p * Wsize_bsize (caml_stat_heap_size));
+    computed_work = 2 * (intnat) (p * Wsize_bsize (caml_stat_heap_size));
   }
   caml_gc_message (0x40, "ordered work = %ld words\n", howmuch);
   caml_gc_message (0x40, "computed work = %ld words\n", computed_work);
@@ -438,7 +485,7 @@ void caml_init_major_heap (asize_t heap_size)
     caml_fatal_error ("Fatal error: not enough memory for the initial heap.\n");
   Chunk_next (caml_heap_start) = NULL;
   caml_heap_end = caml_heap_start + caml_stat_heap_size;
-  Assert ((unsigned long) caml_heap_end % Page_size == 0);
+  Assert ((uintnat) caml_heap_end % Page_size == 0);
 
   caml_stat_heap_chunks = 1;
 
