@@ -6,25 +6,25 @@
 (*          Objective Caml port by John Malecki and Xavier Leroy       *)
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
-(*  Automatique.  Distributed only by permission.                      *)
+(*  en Automatique.  All rights reserved.  This file is distributed    *)
+(*  under the terms of the Q Public License version 1.0.               *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: events.mli,v 1.2 1997/02/19 16:09:20 xleroy Exp $ *)
+(* $Id: events.mli,v 1.4 2005/08/25 15:35:16 doligez Exp $ *)
 
 open Instruct
 
+val get_pos : debug_event -> Lexing.position;;
+
 (** Current events. **)
 
 (* The event at current position. *)
 val current_event : debug_event option ref
 
-(* Recompute the current event *)
-val update_current_event : unit -> unit
-
 (* Current position in source. *)
 (* Raise `Not_found' if not on an event (beginning or end of program). *)
-val current_point : unit -> string * int
+val get_current_event : unit -> debug_event
 
 val current_event_is_before : unit -> bool
 
