@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pattern_matching.ml,v 1.3 1999/11/17 18:57:26 xleroy Exp $ *)
+(* $Id: pattern_matching.ml,v 1.4 2000/12/28 13:03:41 weis Exp $ *)
 
 (************************ Simple pattern matching **********************)
 
@@ -60,7 +60,7 @@ let error_matching () =
 let same_name {qualid = name1} =
   function
     GRname name2 ->
-      (name2 = "") or (name1.id = name2)
+      (name2 = "") || (name1.id = name2)
   | GRmodname name2 ->
       name1 = name2
 
