@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: time_travel.ml,v 1.17 2004/06/21 08:39:32 xleroy Exp $ *)
+(* $Id: time_travel.ml,v 1.21 2006/12/09 16:23:37 ertai Exp $ *)
 
 (**************************** Time travel ******************************)
 
@@ -26,8 +26,10 @@ open Trap_barrier
 open Input_handling
 open Debugger_config
 open Program_loading
+open Question
 
 exception Current_checkpoint_lost
+exception Current_checkpoint_lost_start_at of int64 * int64
 
 let remove_1st key list =
   let rec remove =
@@ -263,7 +265,7 @@ let rec stop_on_event report =
     {rep_type = Breakpoint; rep_program_pointer = pc;
      rep_stack_pointer = sp} ->
       last_breakpoint := Some (pc, sp);
-      update_current_event ();
+      Symbols.update_current_event ();
       begin match !current_event with
         None   -> find_event ()
       | Some _ -> ()
@@ -385,16 +387,21 @@ let forget_process fd pid =
     find (function c -> c.c_pid = pid) (!current_checkpoint::!checkpoints)
   in
     Printf.eprintf "Lost connection with process %d" pid;
-    if checkpoint == !current_checkpoint then begin
-      Printf.eprintf " (active process)\n";
-      match !current_checkpoint.c_state with
-        C_stopped ->
-          Printf.eprintf "at time %Ld" !current_checkpoint.c_time
-      | C_running duration ->
-          Printf.eprintf "between time %Ld and time %Ld"
-                         !current_checkpoint.c_time
-                         (!current_checkpoint.c_time ++ duration)
-      end;
+    let kont =
+      if checkpoint == !current_checkpoint then begin
+        Printf.eprintf " (active process)\n";
+        match !current_checkpoint.c_state with
+          C_stopped ->
+            Printf.eprintf "at time %Ld" !current_checkpoint.c_time;
+            fun () -> raise Current_checkpoint_lost
+        | C_running duration ->
+            Printf.eprintf "between time %Ld and time %Ld"
+                          !current_checkpoint.c_time
+                          (!current_checkpoint.c_time ++ duration);
+            fun () -> raise (Current_checkpoint_lost_start_at
+                              (!current_checkpoint.c_time, duration))
+        end
+      else ignore in
     Printf.eprintf "\n"; flush stderr;
     Input_handling.remove_file fd;
     close_io checkpoint.c_fd;
@@ -403,8 +410,7 @@ let forget_process fd pid =
     checkpoint.c_pid <- -1;             (* Don't exist anymore *)
     if checkpoint.c_parent.c_pid > 0 then
       wait_child checkpoint.c_parent.c_fd;
-    if checkpoint == !current_checkpoint then
-      raise Current_checkpoint_lost
+    kont ()
 
 (* Try to recover when the current checkpoint is lost. *)
 let recover () =
@@ -465,7 +471,7 @@ let rec find_last_breakpoint max_time =
 (* --- Assume 0 <= time < time_max *)
 let rec back_to time time_max =
   let
-    {c_time = t} as checkpoint = find_checkpoint_before (pre64 time_max)
+    {c_time = t} = find_checkpoint_before (pre64 time_max)
   in
     go_to (max time t);
     let (new_time, break) = find_last_breakpoint time_max in
@@ -508,7 +514,7 @@ let step duration =
 
 (* Finish current function. *)
 let finish () =
-  update_current_event ();
+  Symbols.update_current_event ();
   match !current_event with
     None ->
       prerr_endline "`finish' not meaningful in outermost frame.";
@@ -545,7 +551,7 @@ let finish () =
                 done))
 
 let next_1 () =
-  update_current_event ();
+  Symbols.update_current_event ();
   match !current_event with
     None ->                             (* Beginning of the program. *)
       step _1
@@ -553,7 +559,7 @@ let next_1 () =
       let (frame1, pc1) = initial_frame() in
       step _1;
       if not !interrupted then begin
-        update_current_event ();
+        Symbols.update_current_event ();
         match !current_event with
           None -> ()
         | Some event2 ->
@@ -575,7 +581,7 @@ let rec next =
 
 (* Run backward until just before current function. *)
 let start () =
-  update_current_event ();
+  Symbols.update_current_event ();
   match !current_event with
     None ->
       prerr_endline "`start not meaningful in outermost frame.";
@@ -613,7 +619,7 @@ let start () =
       done
 
 let previous_1 () =
-  update_current_event ();
+  Symbols.update_current_event ();
   match !current_event with
     None ->                             (* End of the program. *)
       step _minus1
@@ -621,7 +627,7 @@ let previous_1 () =
       let (frame1, pc1) = initial_frame() in
       step _minus1;
       if not !interrupted then begin
-        update_current_event ();
+        Symbols.update_current_event ();
         match !current_event with
           None -> ()
         | Some event2 ->
