@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ocamlbuild.ml,v 1.1 2007/02/07 08:59:14 ertai Exp $ *)
+(* $Id: ocamlbuild.ml,v 1.1.4.1 2007/11/22 18:34:13 ertai Exp $ *)
 (* Original author: Nicolas Pouillard *)
-My_unix_with_unix.setup ();
+Ocamlbuild_unix_plugin.setup ();
 Ocamlbuild_pack.Main.main ()
