@@ -11,36 +11,35 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: subwindow.c,v 1.4 2001/12/07 13:39:55 xleroy Exp $ */
+/* $Id: subwindow.c,v 1.6 2004/03/24 15:02:05 starynke Exp $ */
 
 #include "libgraph.h"
 
-value gr_open_subwindow(value grx, value gry, value width, value height)
+value caml_gr_open_subwindow(value vx, value vy, value width, value height)
 {
   Window win;
 
   int h = Int_val(height);
   int w = Int_val(width);
-  int x = Int_val(grx);
-  int y = Int_val(gry);
+  int x = Int_val(vx);
+  int y = Int_val(vy);
 
-  gr_check_open();
-  win = XCreateSimpleWindow(grdisplay, grwindow.win,
-                            x, Wcvt(y - h), 
-                            w, h,
-                            0, grblack, grbackground);
-  XMapWindow(grdisplay, win);
-  XFlush(grdisplay);
-  return (id_of_window (win));
+  caml_gr_check_open();
+  win = XCreateSimpleWindow(caml_gr_display, caml_gr_window.win,
+                            x, Wcvt(y + h), w, h,
+                            0, caml_gr_black, caml_gr_background);
+  XMapWindow(caml_gr_display, win);
+  XFlush(caml_gr_display);
+  return (caml_gr_id_of_window (win));
 }
 
-value gr_close_subwindow(value wid)
+value caml_gr_close_subwindow(value wid)
 {
   Window win;
 
-  gr_check_open();
+  caml_gr_check_open();
   sscanf( String_val(wid), "%lu", (unsigned long *)(&win) );
-  XDestroyWindow(grdisplay, win);
-  XFlush(grdisplay);
+  XDestroyWindow(caml_gr_display, win);
+  XFlush(caml_gr_display);
   return Val_unit;
 }
