@@ -12,7 +12,7 @@
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: searchid.mli,v 1.5 2001/12/07 13:40:01 xleroy Exp $ *)
+(* $Id: searchid.mli,v 1.6 2002/07/25 22:51:47 garrigue Exp $ *)
 
 val start_env : Env.t ref
 val module_list : string list ref
@@ -33,7 +33,7 @@ val string_of_kind :  pkind -> string
 exception Error of int * int
 
 val search_string_type :
-      string -> mode:[`exact|`included] -> (Longident.t * pkind) list
+      string -> mode:[`Exact|`Included] -> (Longident.t * pkind) list
 val search_pattern_symbol : string -> (Longident.t * pkind) list
 val search_string_symbol : string -> (Longident.t * pkind) list
 
