@@ -12,7 +12,7 @@
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: setpath.ml,v 1.14 2001/12/07 13:40:01 xleroy Exp $ *)
+(* $Id: setpath.ml,v 1.16 2002/04/26 12:15:34 furuse Exp $ *)
 
 open StdLabels
 open Tk
@@ -57,7 +57,7 @@ let add_to_path ~dirs ?(base="") box =
       begin function
           "." -> base 
         | ".." -> Filename.dirname base
-        | x -> base ^ "/" ^ x
+        | x -> Filename.concat base x
       end
   in
   set_load_path
