@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: str.ml,v 1.19 2004/02/17 10:13:50 xleroy Exp $ *)
+(* $Id: str.ml,v 1.20.6.1 2007/10/31 10:01:29 xleroy Exp $ *)
 
 (** String utilities *)
 
@@ -227,7 +227,9 @@ let compile fold_case re =
   (* Add a new instruction *)
   let emit_instr opc arg =
     if !progpos >= Array.length !prog then begin
-      let nprog = Array.make (2 * Array.length !prog) 0 in
+      let newlen = ref (Array.length !prog) in
+      while !progpos >= !newlen do newlen := !newlen * 2 done;
+      let nprog = Array.make !newlen 0 in
       Array.blit !prog 0 nprog 0 (Array.length !prog);
       prog := nprog
     end;
@@ -667,9 +669,9 @@ and replace_first expr repl text =
 
 let search_forward_progress expr text start =
   let pos = search_forward expr text start in
-  if match_end() = start && start < String.length text
-  then search_forward expr text (start + 1)
-  else pos
+  if match_end() > start then pos
+  else if start < String.length text then search_forward expr text (start + 1)
+  else raise Not_found
 
 let bounded_split expr text num =
   let start =
