@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: posix.c,v 1.55 2007/01/29 12:11:17 xleroy Exp $ */
+/* $Id: posix.c,v 1.55.4.1 2007/11/01 16:42:29 xleroy Exp $ */
 
 /* Thread interface for POSIX 1003.1c threads */
 
@@ -805,7 +805,7 @@ static void decode_sigset(value vset, sigset_t * set)
 {
   sigemptyset(set);
   while (vset != Val_int(0)) {
-    int sig = convert_signal_number(Int_val(Field(vset, 0)));
+    int sig = caml_convert_signal_number(Int_val(Field(vset, 0)));
     sigaddset(set, sig);
     vset = Field(vset, 1);
   }
@@ -822,9 +822,9 @@ static value encode_sigset(sigset_t * set)
 
   Begin_root(res)
     for (i = 1; i < NSIG; i++)
-      if (sigismember(set, i)) {
+      if (sigismember(set, i) > 0) {
         value newcons = alloc_small(2, 0);
-        Field(newcons, 0) = Val_int(i);
+        Field(newcons, 0) = Val_int(caml_rev_convert_signal_number(i));
         Field(newcons, 1) = res;
         res = newcons;
       }
