@@ -11,12 +11,11 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: sockopt.c,v 1.12 2001/12/07 13:40:46 xleroy Exp $ */
+/* $Id: sockopt.c,v 1.15 2002/07/23 14:12:01 doligez Exp $ */
 
 #include <mlvalues.h>
+#include <alloc.h>
 #include "unixsupport.h"
-#include <winsock.h>
-#include <sys/types.h>
 
 static int sockopt_bool[] = {
   SO_DEBUG, SO_BROADCAST, SO_REUSEADDR, SO_KEEPALIVE,
@@ -36,9 +35,9 @@ CAMLprim value getsockopt_int(int *sockopt, value socket,
   int optsize;
 
   optsize = sizeof(optval);
-  if (getsockopt((SOCKET) Handle_val(socket),
+  if (getsockopt(Socket_val(socket),
                  level, sockopt[Int_val(option)],
-		 (void *) &optval, &optsize) == -1)
+                 (void *) &optval, &optsize) == -1)
     uerror("getsockopt", Nothing);
   return Val_int(optval);
 }
@@ -47,9 +46,9 @@ CAMLprim value setsockopt_int(int *sockopt, value socket, int level,
                               value option, value status)
 {
   int optval = Int_val(status);
-  if (setsockopt((SOCKET) Handle_val(socket),
+  if (setsockopt(Socket_val(socket),
                  level, sockopt[Int_val(option)],
-		 (void *) &optval, sizeof(optval)) == -1)
+                 (void *) &optval, sizeof(optval)) == -1)
     uerror("setsockopt", Nothing);
   return Val_unit;
 }
@@ -77,12 +76,12 @@ CAMLprim value getsockopt_optint(int *sockopt, value socket,
 {
   struct linger optval;
   int optsize;
-  value res = Val_int(0);			/* None */
+  value res = Val_int(0);                       /* None */
 
   optsize = sizeof(optval);
-  if (getsockopt((SOCKET) Handle_val(socket),
+  if (getsockopt(Socket_val(socket),
                  level, sockopt[Int_val(option)],
-		 (void *) &optval, &optsize) == -1)
+                 (void *) &optval, &optsize) == -1)
     uerror("getsockopt_optint", Nothing);
   if (optval.l_onoff != 0) {
     res = alloc_small(1, 0);
@@ -99,9 +98,9 @@ CAMLprim value setsockopt_optint(int *sockopt, value socket, int level,
   optval.l_onoff = Is_block (status);
   if (optval.l_onoff)
     optval.l_linger = Int_val (Field (status, 0));
-  if (setsockopt((SOCKET) Handle_val(socket),
+  if (setsockopt(Socket_val(socket),
                  level, sockopt[Int_val(option)],
-		 (void *) &optval, sizeof(optval)) == -1)
+                 (void *) &optval, sizeof(optval)) == -1)
     uerror("setsockopt_optint", Nothing);
   return Val_unit;
 }
@@ -123,9 +122,9 @@ CAMLprim value getsockopt_float(int *sockopt, value socket,
   int optsize;
 
   optsize = sizeof(tv);
-  if (getsockopt((SOCKET) Handle_val(socket),
+  if (getsockopt(Socket_val(socket),
                  level, sockopt[Int_val(option)],
-		 (void *) &tv, &optsize) == -1)
+                 (void *) &tv, &optsize) == -1)
     uerror("getsockopt_float", Nothing);
   return copy_double((double) tv.tv_sec + (double) tv.tv_usec / 1e6);
 }
@@ -139,9 +138,9 @@ CAMLprim value setsockopt_float(int *sockopt, value socket, int level,
   tv_f = Double_val(status);
   tv.tv_sec = (int)tv_f;
   tv.tv_usec = (int) (1e6 * (tv_f - tv.tv_sec));
-  if (setsockopt((SOCKET) Handle_val(socket),
+  if (setsockopt(Socket_val(socket),
                  level, sockopt[Int_val(option)],
-		 (void *) &tv, sizeof(tv)) == -1)
+                 (void *) &tv, sizeof(tv)) == -1)
     uerror("setsockopt_float", Nothing);
   return Val_unit;
 }
