@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: filename.ml,v 1.37 2005/10/25 18:34:07 doligez Exp $ *)
+(* $Id: filename.ml,v 1.41 2007/01/09 13:42:17 doligez Exp $ *)
 
 let generic_quote quotequote s =
   let l = String.length s in
@@ -25,6 +25,24 @@ let generic_quote quotequote s =
   Buffer.add_char b '\'';
   Buffer.contents b
 
+let generic_basename rindex_dir_sep current_dir_name name =
+  let raw_name =
+    try
+      let p = rindex_dir_sep name + 1 in
+      String.sub name p (String.length name - p)
+    with Not_found ->
+      name
+  in
+  if raw_name = "" then current_dir_name else raw_name
+
+let generic_dirname rindex_dir_sep current_dir_name dir_sep name =
+  try
+    match rindex_dir_sep name with
+      0 -> dir_sep
+    | n -> String.sub name 0 n
+  with Not_found ->
+    current_dir_name
+
 module Unix = struct
   let current_dir_name = "."
   let parent_dir_name = ".."
@@ -40,9 +58,11 @@ module Unix = struct
     String.length name >= String.length suff &&
     String.sub name (String.length name - String.length suff)
                     (String.length suff) = suff
-  let temporary_directory =
+  let temp_dir_name =
     try Sys.getenv "TMPDIR" with Not_found -> "/tmp"
   let quote = generic_quote "'\\''"
+  let basename = generic_basename rindex_dir_sep current_dir_name
+  let dirname = generic_dirname rindex_dir_sep current_dir_name dir_sep
 end
 
 module Win32 = struct
@@ -53,7 +73,7 @@ module Win32 = struct
   let rindex_dir_sep s =
     let rec pos i =
       if i < 0 then raise Not_found
-      else if (let c = s.[i] in c = '/' || c = '\\' || c = ':') then i
+      else if is_dir_sep s i then i
       else pos (i - 1)
     in pos (String.length s - 1)
   let is_relative n =
@@ -71,22 +91,46 @@ module Win32 = struct
    (let s = String.sub name (String.length name - String.length suff)
                             (String.length suff) in
     String.lowercase s = String.lowercase suff)
-  let temporary_directory =
+  let temp_dir_name =
     try Sys.getenv "TEMP" with Not_found -> "."
   let quote s =
     let l = String.length s in
     let b = Buffer.create (l + 20) in
     Buffer.add_char b '\"';
-    for i = 0 to l - 1 do
+    let rec loop i =
+      if i = l then () else
       match s.[i] with
-        '\"' -> Buffer.add_string b "\\\""
-      | '\\' -> if i + 1 = l then Buffer.add_string b "\\\\"
-                else if s.[i + 1] = '\"' then Buffer.add_string b "\\\\\\\""
-                else Buffer.add_char b '\\'
-      |   c  -> Buffer.add_char b c
-    done;
+      | '\"' -> loop_bs 0 i;
+      | '\\' -> loop_bs 0 i;
+      | c    -> Buffer.add_char b c; loop (i+1);
+    and loop_bs n i =
+      if i = l then add_bs (2*n) else
+      match s.[i] with
+      | '\"' -> add_bs (2*n+1); Buffer.add_char b '\"'; loop (i+1);
+      | '\\' -> loop_bs (n+1) (i+1);
+      | c    -> add_bs n; loop i
+    and add_bs n = for j = 1 to n do Buffer.add_char b '\\'; done
+    in
+    loop 0;
     Buffer.add_char b '\"';
     Buffer.contents b
+  let has_drive s =
+    let is_letter = function
+      | 'A' .. 'Z' | 'a' .. 'z' -> true
+      | _ -> false
+    in
+    String.length s >= 2 && is_letter s.[0] && s.[1] = ':'
+  let drive_and_path s =
+    if has_drive s
+    then (String.sub s 0 2, String.sub s 2 (String.length s - 2))
+    else ("", s)
+  let dirname s =
+    let (drive, path) = drive_and_path s in
+    let dir = generic_dirname rindex_dir_sep current_dir_name dir_sep path in
+    drive ^ dir
+  let basename s =
+    let (drive, path) = drive_and_path s in
+    generic_basename rindex_dir_sep current_dir_name path
 end
 
 module Cygwin = struct
@@ -98,28 +142,31 @@ module Cygwin = struct
   let is_relative = Win32.is_relative
   let is_implicit = Win32.is_implicit
   let check_suffix = Win32.check_suffix
-  let temporary_directory = Unix.temporary_directory
+  let temp_dir_name = Unix.temp_dir_name
   let quote = Unix.quote
+  let basename = generic_basename rindex_dir_sep current_dir_name
+  let dirname = generic_dirname rindex_dir_sep current_dir_name dir_sep
 end
 
 let (current_dir_name, parent_dir_name, dir_sep, is_dir_sep, rindex_dir_sep,
-     is_relative, is_implicit, check_suffix, temporary_directory, quote) =
+     is_relative, is_implicit, check_suffix, temp_dir_name, quote, basename,
+     dirname) =
   match Sys.os_type with
     "Unix" ->
       (Unix.current_dir_name, Unix.parent_dir_name, Unix.dir_sep,
        Unix.is_dir_sep, Unix.rindex_dir_sep,
        Unix.is_relative, Unix.is_implicit, Unix.check_suffix,
-       Unix.temporary_directory, Unix.quote)
+       Unix.temp_dir_name, Unix.quote, Unix.basename, Unix.dirname)
   | "Win32" ->
       (Win32.current_dir_name, Win32.parent_dir_name, Win32.dir_sep,
        Win32.is_dir_sep, Win32.rindex_dir_sep,
        Win32.is_relative, Win32.is_implicit, Win32.check_suffix,
-       Win32.temporary_directory, Win32.quote)
+       Win32.temp_dir_name, Win32.quote, Win32.basename, Win32.dirname)
   | "Cygwin" ->
       (Cygwin.current_dir_name, Cygwin.parent_dir_name, Cygwin.dir_sep,
        Cygwin.is_dir_sep, Cygwin.rindex_dir_sep,
        Cygwin.is_relative, Cygwin.is_implicit, Cygwin.check_suffix,
-       Cygwin.temporary_directory, Cygwin.quote)
+       Cygwin.temp_dir_name, Cygwin.quote, Cygwin.basename, Cygwin.dirname)
   | _ -> assert false
 
 let concat dirname filename =
@@ -128,24 +175,6 @@ let concat dirname filename =
   then dirname ^ filename
   else dirname ^ dir_sep ^ filename
 
-let basename name =
-  let raw_name =
-    try
-      let p = rindex_dir_sep name + 1 in
-      String.sub name p (String.length name - p)
-    with Not_found ->
-      name
-  in
-  if raw_name = "" then current_dir_name else raw_name
-
-let dirname name =
-  try
-    match rindex_dir_sep name with
-      0 -> dir_sep
-    | n -> String.sub name 0 n
-  with Not_found ->
-    current_dir_name
-
 let chop_suffix name suff =
   let n = String.length name - String.length suff in
   if n < 0 then invalid_arg "Filename.chop_suffix" else String.sub name 0 n
@@ -164,7 +193,7 @@ let prng = Random.State.make_self_init ();;
 
 let temp_file_name prefix suffix =
   let rnd = (Random.State.bits prng) land 0xFFFFFF in
-  concat temporary_directory (Printf.sprintf "%s%06x%s" prefix rnd suffix)
+  concat temp_dir_name (Printf.sprintf "%s%06x%s" prefix rnd suffix)
 ;;
 
 let temp_file prefix suffix =
