@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: obj.ml,v 1.19 2001/12/07 13:40:56 xleroy Exp $ *)
+(* $Id: obj.ml,v 1.23 2004/01/01 16:42:40 doligez Exp $ *)
 
 (* Operations on internal representations of values *)
 
@@ -20,27 +20,37 @@ type t
 external repr : 'a -> t = "%identity"
 external obj : t -> 'a = "%identity"
 external magic : 'a -> 'b = "%identity"
-external is_block : t -> bool = "obj_is_block"
+external is_block : t -> bool = "caml_obj_is_block"
 external is_int : t -> bool = "%obj_is_int"
-external tag : t -> int = "obj_tag"
+external tag : t -> int = "caml_obj_tag"
+external set_tag : t -> int -> unit = "caml_obj_set_tag"
 external size : t -> int = "%obj_size"
 external field : t -> int -> t = "%obj_field"
 external set_field : t -> int -> t -> unit = "%obj_set_field"
-external new_block : int -> int -> t = "obj_block"
-external dup : t -> t = "obj_dup"
-external truncate : t -> int -> unit = "obj_truncate"
+external new_block : int -> int -> t = "caml_obj_block"
+external dup : t -> t = "caml_obj_dup"
+external truncate : t -> int -> unit = "caml_obj_truncate"
 
 let marshal (obj : t) =
   Marshal.to_string obj []
 let unmarshal str pos =
   (Marshal.from_string str pos, pos + Marshal.total_size str pos)
 
-let no_scan_tag = 251
-let closure_tag = 250
-let infix_tag = 249
+let lazy_tag = 246
+let closure_tag = 247
 let object_tag = 248
+let infix_tag = 249
+let forward_tag = 250
+
+let no_scan_tag = 251
+
 let abstract_tag = 251
 let string_tag = 252
 let double_tag = 253
 let double_array_tag = 254
-let final_tag = 255
+let custom_tag = 255
+let final_tag = custom_tag
+
+
+let int_tag = 1000
+let out_of_heap_tag = 1001
