@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parmatch.mli,v 1.9 2003/08/18 08:26:18 garrigue Exp $ *)
+(* $Id: parmatch.mli,v 1.10 2005/03/11 10:12:05 maranget Exp $ *)
 
 (* Detection of partial matches and unused match cases. *)
 open Types
@@ -38,7 +38,13 @@ val lubs : pattern list -> pattern list -> pattern list
 
 val get_mins : ('a -> 'a -> bool) -> 'a list -> 'a list
 
+(* Those to functions recombine one pattern and its arguments:
+   For instance:
+     (_,_)::p1::p2::rem -> (p1, p2)::rem
+   The second one will replace mutable arguments by '_'
+*)
 val set_args : pattern -> pattern list -> pattern list
+val set_args_erase_mutable : pattern -> pattern list -> pattern list
 
 val pat_of_constr : pattern -> constructor_description -> pattern
 val complete_constrs :
