@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedtree.mli,v 1.35 2006/04/05 02:28:13 garrigue Exp $ *)
+(* $Id: typedtree.mli,v 1.35.8.2 2007/07/10 07:34:35 garrigue Exp $ *)
 
 (* Abstract syntax tree after typing *)
 
@@ -32,10 +32,10 @@ and pattern_desc =
   | Tpat_constant of constant
   | Tpat_tuple of pattern list
   | Tpat_construct of constructor_description * pattern list
-  | Tpat_variant of label * pattern option * row_desc
+  | Tpat_variant of label * pattern option * row_desc ref
   | Tpat_record of (label_description * pattern) list
   | Tpat_array of pattern list
-  | Tpat_or of pattern * pattern * Path.t option
+  | Tpat_or of pattern * pattern * row_desc option
 
 type partial = Partial | Total
 type optional = Required | Optional
